import chimera, Tkinter as tk
from chimera.baseDialog import ModelessDialog

class WidgetBuffet(ModelessDialog):
    buttons = ("Apply", "Close")
    help = False    # Leave this out to get a Help button.
    title = "Window title goes here"

    def fillInUI(self, parent):
        # Text to be displayed in the dialog window:
        topText = tk.Label(parent, text =  "Text at top of dialog",
                           font = ("Helvetica", 11, "normal"))
        topText.grid(row = 0, column = 0, columnspan = 2, pady = 4, sticky = tk.W)

        #-------------------------------------------------------------------------
        # Setting up two check boxes:
        # Checking the box will change the value of the IntVar and will initiate
        # the execution of the function specified by the command option.
        checkBoxFrame = tk.LabelFrame(parent, text = "Title for Checkboxes", bd = 2, relief = tk.RIDGE)
        checkBoxFrame.grid(row = 1, column = 0, padx = 4, pady = 4, sticky = tk.W)
        
        self.checkedBoxA = tk.IntVar()
        cbA = tk.Checkbutton(checkBoxFrame, text = "Checkbox A   ", variable = self.checkedBoxA,
                             command = self.boxAfunc, indicatoron = 1)
        cbA.grid(row = 0, column = 0, pady = 6, sticky = tk.E)
        cbA.select()    # This button initialized with a check mark.

        self.checkedBoxB = tk.IntVar()
        cbB = tk.Checkbutton(checkBoxFrame, text = "Checkbox B   ", variable = self.checkedBoxB,
                             command = self.boxBfunc, indicatoron = 1)
        cbB.grid(row = 1, column = 0, pady = 6, sticky = tk.E)

        #-------------------------------------------------------------------------
        # Setting up two buttons (one disabled for now):
        buttonsFrame = tk.LabelFrame(parent, text = "Title for buttons", bd = 2, relief = tk.RIDGE)
        buttonsFrame.grid(row = 1, column = 1, padx = 4, pady = 4, sticky = tk.E)
        
        self.btnA = tk.Button(buttonsFrame, text = " Button A name ",
                              command = lambda: self.btnFunc("A"))
        self.btnA.grid(row = 0, column = 0, padx = 4, pady = 4)

        self.btnB = tk.Button(buttonsFrame, text = " Button B name ",
                              command = lambda: self.btnFunc("B"), state = tk.DISABLED)
        self.btnB.grid(row = 1, column = 0, padx = 4, pady = 4)

        #-------------------------------------------------------------------------
        # Set up radio buttons as specified by a list:
        rButtonsFrame = tk.LabelFrame(parent, text = "Title for radio buttons", bd = 2, relief = tk.RIDGE)
        rButtonsFrame.grid(row = 2, column = 0, columnspan = 2, padx = 4, pady = 4, sticky = tk.W + tk.E)
        
        self.rBtnResult = tk.StringVar() # Use tk.IntVar() here for integer result values.
        radioBtn_L = [("Name A", "Result A", 0, 0), ("Name B", "Result B", 0, 1),
                      ("Name C", "Result C", 0, 2), ("Name D", "Result D", 0, 3),
                      ("Name E", "Result E", 1, 0), ("Name F", "Result F", 1, 1),
                      ("Name G", "Result G", 1, 2), ("Name H", "Result H", 1, 3)]
        
        self.rBtnResult.set("Result A") # This button starts off selected.
        for text, rslt, rw, cl in radioBtn_L:
            rBtn = tk.Radiobutton(rButtonsFrame, text = text, variable = self.rBtnResult,
                                  value = rslt, command = self.rBtnfunc)
            rBtn.grid(row = rw, column = cl, padx = 4, sticky = tk.W)
            
        #-------------------------------------------------------------------------
        # Set up list box:
        listBoxFrame = tk.LabelFrame(parent, text = "Title for list box", bd = 2, relief = tk.RIDGE)
        listBoxFrame.grid(row = 3, column = 0, padx = 4, pady = 4, sticky = tk.W + tk.NS)
        
        self.lstBx = tk.Listbox(listBoxFrame, height = 5, width = 18,
                                selectmode = tk.SINGLE, bg = "white" )
                                # Use MULTIPLE for checkbox like operation.
        
        scroll = tk.Scrollbar(listBoxFrame, command = self.lstBx.yview)
        self.lstBx.configure(yscrollcommand = scroll.set)
        scroll.grid(row = 0, column = 1, padx = 5, pady = 4, sticky = tk.W +tk.N + tk.S)
        
        lstBx_L = ["Entry 0", "Entry 1", "Entry 2", "Entry 3", "Entry 4", "Entry 5"]
        for item in lstBx_L: self.lstBx.insert(tk.END, item)
        self.lstBx.grid(row = 0, column = 0, padx = 4, pady = 4, sticky = tk.E)
        self.lstBx.bind("<Double-Button-1>", self.Apply)

        #-------------------------------------------------------------------------
        # Set up a canvas to the right of the list box:
        canvasFrame = tk.Frame(parent, bd = 2, relief = tk.RIDGE)
        canvasFrame.grid(row = 3, column = 1, padx = 4, pady = 4, sticky = tk.E)

        # Title left out (a picture is worth a thousand words...)

        picture = tk.Canvas(canvasFrame, width = 110, height = 107)
        picture.grid(row = 0, column = 0)

        gfImage = tk.PhotoImage(file = "canvasExample.gif")
        picture.create_image(0, 0, image = gfImage, anchor = tk.NW)
        # Store a reference to the image as an attribute of the canvas:
        picture.img = gfImage  # Remove this and the image disappears.

        #-------------------------------------------------------------------------
        # Setting up a scale:
        scaleFrame = tk.LabelFrame(parent, text = "Title for scale", bd = 2, relief = tk.RIDGE)
        scaleFrame.grid(row = 4, column = 0, padx = 4, pady = 4, sticky = tk.W)
        
        self.scaleVar = tk.DoubleVar()
        scale = tk.Scale(scaleFrame, variable = self.scaleVar, orient = tk.HORIZONTAL,
                         length = 100, from_ = 1.0, to = 12.0, resolution = 0.1,
                         troughcolor = "goldenrod1")
        scale.grid(row = 0, column = 0, sticky = tk.W)
        
        acceptButton = tk.Button(scaleFrame, text = "Accept", command = self.acceptSelection)
        acceptButton.grid(row = 0, column = 1, padx = 4, pady = 4, sticky = tk.S + tk.W)

        #-------------------------------------------------------------------------
        # Setting up a entry box:
        entryFrame = tk.LabelFrame(parent, text = "Title for entry box", bd = 2, relief = tk.RIDGE)
        entryFrame.grid(row = 4, column = 1, padx = 4, pady = 4, sticky = tk.E + tk.NS)
        
        self.entryVar = tk.StringVar()
        entry = tk.Entry(entryFrame, textvariable = self.entryVar, width = 14, bg = "white")
        entry.grid(row = 0, column = 0, padx = 4, pady = 4, sticky = tk.E)
        #entry.config(state = tk.DISABLED)  # Use this if the entry box is to start DISABLED.

        #-------------------------------------------------------------------------
        # Setting up a text box:
        textFrame = tk.LabelFrame(parent, text = "Title for text box", bd = 2, relief = tk.RIDGE)
        textFrame.grid(row = 5, column = 0, columnspan = 2,
                      padx = 4, pady = 4, sticky = tk.W + tk.E)
        
        self.textWidget = tk.Text(textFrame, width = 40, height = 6,
                                  wrap = tk.WORD, bg = "white")
        self.textWidget.grid(row = 0, column = 0, padx = 4, sticky = tk.W)

        textScroll = tk.Scrollbar(textFrame, command = self.textWidget.yview)
        self.textWidget.configure(yscrollcommand = textScroll.set)
        textScroll.grid(row = 0, column = 1, padx = 2, pady = 4,
                        sticky = tk.W +tk.N + tk.S)
     
    #=============================================================================
    # CALLBACK functions:
    #-------------------------------------------------------------------------
    # Callback functions needed for check boxes:
    def boxAfunc(self):
        line = "Box A checked. Value = %1d" % self.checkedBoxA.get()
        self.textWidget.insert(tk.END, line + "\n"); self.textWidget.yview(tk.END)

    def boxBfunc(self):
        line = "Box B checked. Value = %1d" % self.checkedBoxB.get()
        self.textWidget.insert(tk.END, line + "\n"); self.textWidget.yview(tk.END)

    #-------------------------------------------------------------------------
    # Callback functions needed for Buttons:
    def btnFunc(self, buttonSpec):
        line = "Button " + buttonSpec + " depressed."
        self.textWidget.insert(tk.END, line + "\n"); self.textWidget.yview(tk.END)
        
        # The next line shows how button B can be "enabled".
        # Usually, this would be done by the mainline program...
        self.btnB.configure(state = tk.NORMAL)

    #-------------------------------------------------------------------------
    # Callback function needed for radio buttons:
    def rBtnfunc(self):
        line = "Radio button: " + self.rBtnResult.get() + " selected."
        self.textWidget.insert(tk.END, line + "\n"); self.textWidget.yview(tk.END)
        
    #-------------------------------------------------------------------------
    # Callback function needed for Scale widget:
    def acceptSelection(self):
        line = "Scale value = %4.1f  " % self.scaleVar.get()
        self.textWidget.insert(tk.END, line + "\n"); self.textWidget.yview(tk.END)

    #-------------------------------------------------------------------------
    # Callback function needed for Apply button:
    def Apply(self, event = None):
        # For "something to do" with Apply we get the selected list box item
        # and the text in the entry box.
        line = "Apply key depressed. List box selection: " + str(self.lstBx.curselection())
        line = line + " Entry box contains: " + self.entryVar.get()
        self.textWidget.insert(tk.END, line + "\n"); self.textWidget.yview(tk.END)
    
#===============================================================================
# Mainline for testing:        
wb = WidgetBuffet()
wb.textWidget.insert(tk.END, "Using the widgets should show results in this text box.\n")

